import {Box, TextField, Button, Typography, FormControl, InputLabel, Select, MenuItem} from '@mui/material'
import {Create} from '@refinedev/mui'
import {useForm} from '@refinedev/react-hook-form'
import {useState} from 'react'

import {MEDIA_TYPES} from '../../lib/constants'

export const MediaCreate = () => {
  const {
    saveButtonProps,
    refineCore: {formLoading, onFinish},
    register,
    formState: {errors},
    setValue,
    handleSubmit,
    watch,
  } = useForm({})

  const [selectedFile, setSelectedFile] = useState<File | null>(null)
  const [previewUrl, setPreviewUrl] = useState<string | null>(null)

  const handleFileChange = (event: React.ChangeEvent<HTMLInputElement>) => {
    const file = event.target.files?.[0]
    if (file) {
      setValue('file', file)
      setSelectedFile(file)

      if (file.type.startsWith('image/')) {
        const url = URL.createObjectURL(file)
        setPreviewUrl(url)
      } else {
        setPreviewUrl(null)
      }
    }
  }

  return (
    <Create
      isLoading={formLoading}
      saveButtonProps={{
        ...saveButtonProps,
        onClick: handleSubmit((data) => {
          const formData = new FormData()

          formData.append('filename', data.filename)
          formData.append('type', String(data.media_type))

          const file = watch('file')
          if (file) {
            formData.append('file', file)
          }

          onFinish(formData)
        }),
      }}
    >
      <Box component="form" sx={{display: 'flex', flexDirection: 'column'}} autoComplete="off">
        <Box display="flex" flexDirection="column-reverse" alignItems="center" gap={6}>
          <Box display="flex" alignItems="center" gap={2}>
            <Button variant="contained" component="label">
              {selectedFile ? 'Изменить файл' : 'Загрузить файл'}
              <input type="file" hidden onChange={handleFileChange} />
            </Button>

            {selectedFile && (
              <Typography variant="body2" color="text.secondary">
                {selectedFile.name}
              </Typography>
            )}
          </Box>

          {previewUrl && (
            <Box mt={2} display="flex" justifyContent="center">
              <img src={previewUrl} alt="Preview" style={{maxWidth: '200px', borderRadius: 8}} />
            </Box>
          )}
        </Box>

        <TextField
          {...register('filename', {
            required: 'Это поле является обязательным',
          })}
          error={!!(errors as any)?.filename}
          helperText={(errors as any)?.filename?.message}
          margin="normal"
          fullWidth
          InputLabelProps={{shrink: true}}
          type="text"
          label="Название"
          name="filename"
        />

        <FormControl fullWidth margin="normal" error={!!errors.media_type}>
          <InputLabel id="media-type-label">Тип</InputLabel>
          <Select
            labelId="media-type-label"
            label="Тип"
            {...register('media_type', {
              required: 'Это поле является обязательным',
              valueAsNumber: true,
            })}
          >
            {MEDIA_TYPES.map((type) => (
              <MenuItem key={type.value} value={type.value}>
                {type.label}
              </MenuItem>
            ))}
          </Select>
          {errors.media_type && (
            <Typography variant="caption" color="error">
              {!!(errors as any)?.message}
            </Typography>
          )}
        </FormControl>
      </Box>
    </Create>
  )
}
